
package edu.sc.seis.seisFile.fdsnws;

import java.time.Instant;

import edu.sc.seis.seisFile.ChannelTimeWindow;
import edu.sc.seis.seisFile.TimeUtils;
import edu.sc.seis.seisFile.fdsnws.stationxml.Channel;

/** Autogenerated by groovy FDSNQueryParamGenerator.groovy in src/metacode/groovy
 */
public class FDSNDataSelectQueryParams extends AbstractQueryParams implements Cloneable {

    public FDSNDataSelectQueryParams() {
        this(DEFAULT_HOST);
    }

    public FDSNDataSelectQueryParams(String host) {
        super(host==null ? DEFAULT_HOST : host);
    }

    public FDSNDataSelectQueryParams clone() {
        FDSNDataSelectQueryParams out = new FDSNDataSelectQueryParams(getHost());
        out.cloneNonParams(this);
        for (String key : params.keySet()) {
            out.setParam(key, params.get(key));
        }
        return out;
    }

    public FDSNDataSelectQueryParams setHost(String host) {
        this.host = host;
        return this;
    }
    public FDSNDataSelectQueryParams setPort(int port) {
        this.port = port;
        return this;
    }


    public static final String STARTTIME = "starttime";

    public static final String STARTTIME_SHORT = "start";


    public FDSNDataSelectQueryParams clearStartTime() {
        clearParam(STARTTIME);
        return this;
    }

    /** Limit results to time series samples on or after the specified start time
     *  @param value value to set
     *  @return the queryParams for method chaining
     */
    public FDSNDataSelectQueryParams setStartTime(Instant value) {
        setParam(STARTTIME, value);
        return this;
    }


    public static final String ENDTIME = "endtime";

    public static final String ENDTIME_SHORT = "end";


    public FDSNDataSelectQueryParams clearEndTime() {
        clearParam(ENDTIME);
        return this;
    }

    /** Limit results to time series samples on or before the specified end time
     *  @param value value to set
     *  @return the queryParams for method chaining
     */
    public FDSNDataSelectQueryParams setEndTime(Instant value) {
        setParam(ENDTIME, value);
        return this;
    }


    public static final String NETWORK = "network";

    public static final String NETWORK_SHORT = "net";


    public FDSNDataSelectQueryParams clearNetwork() {
        clearParam(NETWORK);
        return this;
    }


    /** Select one or more network codes. Can be SEED network codes or data center defined codes. Multiple codes are comma-separated.
     */
    public FDSNDataSelectQueryParams setNetwork(String[] value) {
        clearNetwork();
        for(String v: value) appendToNetwork(v);
        return this;
    }

    public FDSNDataSelectQueryParams appendToNetwork(String value) {
        appendToParam(NETWORK, value);
        return this;
    }


    public static final String STATION = "station";

    public static final String STATION_SHORT = "sta";


    public FDSNDataSelectQueryParams clearStation() {
        clearParam(STATION);
        return this;
    }


    /** Select one or more SEED station codes. Multiple codes are comma-separated.
     */
    public FDSNDataSelectQueryParams setStation(String[] value) {
        clearStation();
        for(String v: value) appendToStation(v);
        return this;
    }

    public FDSNDataSelectQueryParams appendToStation(String value) {
        appendToParam(STATION, value);
        return this;
    }


    public static final String LOCATION = "location";

    public static final String LOCATION_SHORT = "loc";


    public FDSNDataSelectQueryParams clearLocation() {
        clearParam(LOCATION);
        return this;
    }


    /** Select one or more SEED location identifiers. Multiple identifiers are comma-separated. As a special case -- (two dashes) will be translated to a string of two space characters to match blank location IDs.
     */
    public FDSNDataSelectQueryParams setLocation(String[] value) {
        clearLocation();
        for(String v: value) appendToLocation(v);
        return this;
    }

    public FDSNDataSelectQueryParams appendToLocation(String value) {
        if (value == null || Channel.EMPTY_LOC_CODE.equals(value.trim())) { value = "--";}
        appendToParam(LOCATION, value);
        return this;
    }


    public static final String CHANNEL = "channel";

    public static final String CHANNEL_SHORT = "cha";


    public FDSNDataSelectQueryParams clearChannel() {
        clearParam(CHANNEL);
        return this;
    }


    /** Select one or more SEED channel codes. Multiple codes are comma-separated.
     */
    public FDSNDataSelectQueryParams setChannel(String[] value) {
        clearChannel();
        for(String v: value) appendToChannel(v);
        return this;
    }

    public FDSNDataSelectQueryParams appendToChannel(String value) {
        appendToParam(CHANNEL, value);
        return this;
    }


    public static final String QUALITY = "quality";



    public FDSNDataSelectQueryParams clearQuality() {
        clearParam(QUALITY);
        return this;
    }

    /** Select a specific SEED quality indicator, handling is data center dependent.
     *  @param value value to set
     *  @return the queryParams for method chaining
     */
    public FDSNDataSelectQueryParams setQuality(String value) {
        setParam(QUALITY, value);
        return this;
    }


    public static final String MINIMUMLENGTH = "minimumlength";



    public FDSNDataSelectQueryParams clearMinimumLength() {
        clearParam(MINIMUMLENGTH);
        return this;
    }

    /** Limit results to continuous data segments of a minimum length specified in seconds.
     *  @param value value to set
     *  @return the queryParams for method chaining
     */
    public FDSNDataSelectQueryParams setMinimumLength(int value) {
        setParam(MINIMUMLENGTH, value);
        return this;
    }


    public static final String LONGESTONLY = "longestonly";



    public FDSNDataSelectQueryParams clearLongestOnly() {
        clearParam(LONGESTONLY);
        return this;
    }

    /** Limit results to the longest continuous segment per channel.
     *  @param value value to set
     *  @return the queryParams for method chaining
     */
    public FDSNDataSelectQueryParams setLongestOnly(boolean value) {
        setParam(LONGESTONLY, value);
        return this;
    }


    

    /**
     * Forms the list of ChannelTimeWindow for use in a POST request to
     * the web service. All possible combinations of Networks, Stations
     * and Channels are combined with the start and end times to form
     * the list of ChannelTimeWindow.
     *
     * @return List of Channels paired with the time window
     */
    public java.util.List<ChannelTimeWindow> createChannelTimeWindow() {
        java.util.List<ChannelTimeWindow> request = new java.util.ArrayList<ChannelTimeWindow>();
        String[] netSplit = getParam(NETWORK).split(",");
        String[] staSplit = getParam(STATION).split(",");
        String[] locSplit = getParam(LOCATION).split(",");
        String[] chanSplit = getParam(CHANNEL).split(",");
        Instant beginDate = TimeUtils.parseISOString(getParam(STARTTIME));
        Instant endDate = TimeUtils.parseISOString(getParam(ENDTIME));
            for (int n = 0; n < netSplit.length; n++) {
                for (int s = 0; s < staSplit.length; s++) {
                    for (int l = 0; l < locSplit.length; l++) {
                        for (int c = 0; c < chanSplit.length; c++) {
                            request.add(new ChannelTimeWindow(netSplit[n],
                                                              staSplit[s],
                                                              locSplit[l],
                                                              chanSplit[c],
                                                              beginDate,
                                                              endDate));
                        }
                    }
                }
            }
        return request;
    }

    public String formPostString() {
        return formPostString(createChannelTimeWindow());
    }

    /**
     * Forms the text for use in a POST request to the web service. Channel and
     * time window are taken from the list of ChannelTimeWindow.
     *
     * @return
     */
    public String formPostString(java.util.List<ChannelTimeWindow> request) {
        StringBuffer out = new StringBuffer();
        if (getParam(QUALITY) != null) {
            out.append(QUALITY + "=" + getParam(QUALITY) + NEWLINE);
        }
        if (getParam(MINIMUMLENGTH) != null) {
            out.append(MINIMUMLENGTH + "=" + getParam(MINIMUMLENGTH) + NEWLINE);
        }
        if (getParam(LONGESTONLY) != null) {
            out.append(LONGESTONLY + "=" + getParam(LONGESTONLY) + NEWLINE);
        }
        String SEP = " ";
        for (ChannelTimeWindow ctw : request) {
            out.append(ctw.formString(SEP, createDateFormat(), true)+NEWLINE);
        }
        return out.toString();
    }

    @Override
    public String getServiceName() {
        return DATASELECT_SERVICE;
    }

    public static final String DATASELECT_SERVICE = "dataselect";



}

